package com.sun.jmx.remote.security;

import java.util.ArrayList;
import java.util.List;
import java.util.HashMap;
import java.util.regex.Pattern;
import java.util.StringTokenizer;
import java.util.Iterator;
import java.util.Set;
import java.security.Principal;
import java.security.AccessControlContext;
import java.security.PrivilegedAction;
import java.security.AccessController;
import javax.security.auth.Subject;
import java.io.FileInputStream;
import javax.management.ObjectName;
import javax.management.MBeanServer;
import java.io.IOException;
import java.util.Properties;
import java.util.Map;

public class MBeanServerFileAccessController extends MBeanServerAccessController
{
    static final String READONLY = "readonly";
    static final String READWRITE = "readwrite";
    static final String CREATE = "create";
    static final String UNREGISTER = "unregister";
    private Map<String, MBeanServerFileAccessController.Access> accessMap;
    private Properties originalProps;
    private String accessFileName;
    
    public MBeanServerFileAccessController(String accessFileName) throws IOException {
        this.accessFileName = accessFileName;
        this.parseProperties(propertiesFromFile(accessFileName));
    }
    
    public MBeanServerFileAccessController(String s, MBeanServer mBeanServer) throws IOException {
        this(s);
        this.setMBeanServer(mBeanServer);
    }
    
    public MBeanServerFileAccessController(Properties originalProps) throws IOException {
        if (originalProps == null) {
            throw new IllegalArgumentException("Null properties");
        }
        this.parseProperties(this.originalProps = originalProps);
        return;
    }
    
    public MBeanServerFileAccessController(Properties properties, MBeanServer mBeanServer) throws IOException {
        this(properties);
        this.setMBeanServer(mBeanServer);
    }
    
    public void checkRead() {
        this.checkAccess(MBeanServerFileAccessController.AccessType.READ, null);
    }
    
    public void checkWrite() {
        this.checkAccess(MBeanServerFileAccessController.AccessType.WRITE, null);
    }
    
    public void checkCreate(String s) {
        this.checkAccess(MBeanServerFileAccessController.AccessType.CREATE, s);
    }
    
    public void checkUnregister(ObjectName objectName) {
        this.checkAccess(MBeanServerFileAccessController.AccessType.UNREGISTER, null);
    }
    
    public synchronized void refresh() throws IOException {
        Properties properties;
        if (this.accessFileName == null) {
            properties = this.originalProps;
        }
        else {
            properties = propertiesFromFile(this.accessFileName);
        }
        this.parseProperties(properties);
    }
    
    private static Properties propertiesFromFile(String name) throws IOException {
        FileInputStream inStream = new FileInputStream(name);
        try {
            Properties properties = new Properties();
            properties.load(inStream);
            return properties;
        }
        finally {
            inStream.close();
        }
    }
    
    private synchronized void checkAccess(MBeanServerFileAccessController.AccessType accessType, String str) {
        Subject subject = (Subject)AccessController.doPrivileged(new PrivilegedAction<Subject>() {
            
            @Override
            public Subject run() {
                return Subject.getSubject(acc);
            }
        });
        if (subject == null) {
            return;
        }
        Set<Principal> principals = subject.getPrincipals();
        String string = null;
        Iterator<Object> iterator = principals.iterator();
        while (iterator.hasNext()) {
            MBeanServerFileAccessController.Access access = (MBeanServerFileAccessController.Access)this.accessMap.get(((Principal)iterator.next()).getName());
            if (access != null) {
                int n;
                switch (accessType) {
                    case READ: {
                        n = true;
                        break;
                    }
                    case WRITE: {
                        n = access.write;
                        break;
                    }
                    case UNREGISTER: {
                        n = access.unregister;
                        if (n == 0 && access.write) {
                            string = "unregister";
                            break;
                        }
                        break;
                    }
                    case CREATE: {
                        n = checkCreateAccess(access, str);
                        if (n == 0 && access.write) {
                            string = "create " + str;
                            break;
                        }
                        break;
                    }
                    default: {
                        throw new AssertionError();
                    }
                }
                if (!(n == 0)) {
                    return;
                }
                continue;
            }
            else {
                continue;
            }
        }
        SecurityException o = new SecurityException("Access denied! Invalid access level for requested MBeanServer operation.");
        if (string != null) {
            o.initCause(new SecurityException("Access property for this identity should be similar to: readwrite " + string));
        }
        throw o;
    }
    
    private static boolean checkCreateAccess(MBeanServerFileAccessController.Access access, String s) {
        String[] createPatterns = access.createPatterns;
        int length = createPatterns.length;
        int i = 0;
        while (i < length) {
            if (classNameMatch(createPatterns[i], s)) {
                return true;
            }
            ++i;
        }
        return false;
    }
    
    private static boolean classNameMatch(String str, String s) {
        StringBuilder sb = new StringBuilder();
        StringTokenizer stringTokenizer = new StringTokenizer(str, "*", true);
        while (stringTokenizer.hasMoreTokens()) {
            String nextToken = stringTokenizer.nextToken();
            if (nextToken.equals("*")) {
                sb.append("[^.]*");
            }
            else {
                sb.append(Pattern.quote(nextToken));
            }
        }
        return s.matches(sb.toString());
    }
    
    private void parseProperties(Properties properties) {
        this.accessMap = new HashMap();
        for (Map.Entry entry : properties.entrySet()) {
            String s = (String)entry.getKey();
            this.accessMap.put(s, MBeanServerFileAccessController.Parser.parseAccess(s, (String)entry.getValue()));
        }
    }
    
    private enum AccessType
    {
        READ, 
        WRITE, 
        CREATE, 
        UNREGISTER;
    }
    
    private static class Access
    {
        final boolean write;
        final String[] createPatterns;
        private boolean unregister;
        private final String[] NO_STRINGS;
        
        Access(boolean write, boolean unregister, List<String> list) {
            this.NO_STRINGS = new String[0];
            this.write = write;
            int n = (list == null) ? 0 : list.size();
            if (n == 0) {
                this.createPatterns = this.NO_STRINGS;
            }
            else {
                this.createPatterns = (String[])list.toArray(new String[n]);
            }
            this.unregister = unregister;
        }
    }
    
    private static class Parser
    {
        private static final int EOS = -1;
        private final String identity;
        private final String s;
        private final int len;
        private int i;
        private int c;
        
        private Parser(String identity, String s) {
            this.identity = identity;
            this.s = s;
            this.len = s.length();
            this.i = 0;
            if (this.i < this.len) {
                this.c = s.codePointAt(this.i);
            }
            else {
                this.c = -1;
            }
        }
        
        static MBeanServerFileAccessController.Access parseAccess(String s, String s2) {
            return new Parser(s, s2).parseAccess();
        }
        
        private MBeanServerFileAccessController.Access parseAccess() {
            this.skipSpace();
            String word = this.parseWord();
            MBeanServerFileAccessController.Access readWrite;
            if (word.equals("readonly")) {
                readWrite = new MBeanServerFileAccessController.Access(false, false, null);
            }
            else if (word.equals("readwrite")) {
                readWrite = this.parseReadWrite();
            }
            else {
                throw this.syntax("Expected readonly or readwrite: " + word);
            }
            if (this.c != -1) {
                throw this.syntax("Extra text at end of line");
            }
            return readWrite;
        }
        
        private MBeanServerFileAccessController.Access parseReadWrite() {
            ArrayList<String> list = new ArrayList();
            boolean n = false;
            while (true) {
                this.skipSpace();
                if (this.c == -1) {
                    return new MBeanServerFileAccessController.Access(true, (boolean)n, list);
                }
                String word = this.parseWord();
                if (word.equals("unregister")) {
                    n = true;
                }
                else if (word.equals("create")) {
                    this.parseCreate(list);
                }
                else {
                    throw this.syntax("Unrecognized keyword " + word);
                }
            }
        }
        
        private void parseCreate(List<String> list) {
            while (true) {
                this.skipSpace();
                list.add(this.parseClassName());
                this.skipSpace();
                if (this.c == 44) {
                    this.next();
                }
                else {
                    break;
                }
            }
        }
        
        private String parseClassName() {
            int i = this.i;
            int n = 0;
            while (true) {
                if (this.c == 46) {
                    if (n == 0) {
                        throw this.syntax("Bad . in class name");
                    }
                    n = 0;
                }
                else if (this.c == 42 || Character.isJavaIdentifierPart(this.c)) {
                    n = 1;
                }
                else {
                    String substring = this.s.substring(i, this.i);
                    if (n == 0) {
                        throw this.syntax("Bad class name " + substring);
                    }
                    return substring;
                }
                this.next();
            }
        }
        
        private void next() {
            if (this.c != -1) {
                this.i += Character.charCount(this.c);
                if (this.i < this.len) {
                    this.c = this.s.codePointAt(this.i);
                }
                else {
                    this.c = -1;
                }
            }
        }
        
        private void skipSpace() {
            while (Character.isWhitespace(this.c)) {
                this.next();
            }
        }
        
        private String parseWord() {
            this.skipSpace();
            if (this.c == -1) {
                throw this.syntax("Expected word at end of line");
            }
            int i = this.i;
            while (this.c != -1 && !Character.isWhitespace(this.c)) {
                this.next();
            }
            String substring = this.s.substring(i, this.i);
            this.skipSpace();
            return substring;
        }
        
        private IllegalArgumentException syntax(String str) {
            return new IllegalArgumentException(str + " [" + this.identity + " " + this.s + "]");
        }
        
        static {
            assert !Character.isWhitespace(-1);
            return;
        }
    }
}
